@@ -4,7 +4,7 @@
  * created by Marc Ewing.
  * Currently maintained by Andrew G. Morgan <morgan@linux.kernel.org>
  *
- * $Id: pam_handlers.c,v 1.1.1.1 2000/06/20 22:11:17 agmorgan Exp $
+ * $Id: pam_handlers.c,v 1.2 2001/01/22 06:07:28 agmorgan Exp $
  *
  */
 
@@ -399,7 +399,7 @@ int _pam_init_handlers(pam_handle_t *pamh)
  * preceeded by lines of comments and also extended with "\\\n"
  */
 
-int _pam_assemble_line(FILE *f, char *buffer, int buf_len)
+static int _pam_assemble_line(FILE *f, char *buffer, int buf_len)
 {
     char *p = buffer;
     char *s, *os;
