@@ -1,7 +1,7 @@
 /* pam_mail module */
 
 /*
- * $Id: pam_env.c,v 1.3 1999/11/08 05:46:53 morgan Exp $
+ * $Id: pam_env.c,v 1.2 2000/11/19 23:54:03 agmorgan Exp $
  *
  * Written by Dave Kinchlea <kinch@kinch.ark.com> 1997/01/31
  * Inspired by Andrew Morgan <morgan@parc.power.net, who also supplied the 
@@ -15,10 +15,8 @@
 #define DEFAULT_ETC_ENVFILE     "/etc/environment"
 #define DEFAULT_READ_ENVFILE    1
 
-#define _GNU_SOURCE
-#define _BSD_SOURCE
+#include <security/_pam_aconf.h>
 
-#include <features.h>
 #include <ctype.h>
 #include <errno.h>
 #include <pwd.h>
