@@ -1,14 +1,11 @@
 /*
- * $Id: pam_filter.c,v 1.2 1999/07/08 05:01:45 morgan Exp $
+ * $Id: pam_filter.c,v 1.2 2000/11/19 23:54:03 agmorgan Exp $
  *
  * written by Andrew Morgan <morgan@transmeta.com> with much help from
  * Richard Stevens' UNIX Network Programming book.
  */
 
-#ifdef linux
-# define _GNU_SOURCE
-# include <features.h>
-#endif
+#include <security/_pam_aconf.h>
 
 #include <stdlib.h>
 #include <syslog.h>
