@@ -1,7 +1,7 @@
 /* pam_lastlog module */
 
 /*
- * $Id: pam_lastlog.c,v 1.2 1999/07/08 05:01:51 morgan Exp $
+ * $Id: pam_lastlog.c,v 1.2 2000/11/19 23:54:04 agmorgan Exp $
  *
  * Written by Andrew Morgan <morgan@linux.kernel.org> 1996/3/11
  *
@@ -10,10 +10,7 @@
  * present (login) service.
  */
 
-#ifdef linux
-# define _GNU_SOURCE
-# include <features.h>
-#endif
+#include <security/_pam_aconf.h>
 
 #include <fcntl.h>
 #include <time.h>
@@ -28,6 +25,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
+#define __USE_BSD
 #include <syslog.h>
 #include <unistd.h>
 
