@@ -1,5 +1,5 @@
 /* 
- * $Id: support.c,v 1.4 2000/11/25 04:40:55 agmorgan Exp $
+ * $Id: support.c,v 1.7 2000/12/20 05:15:05 vorlon Exp $
  *
  * Copyright information at end of file.
  */
@@ -27,12 +27,23 @@ extern char *bigcrypt(const char *key, const char *salt);
 
 /* syslogging function for errors and other information */
 
-void _log_err(int err, const char *format,...)
+void _log_err(int err, pam_handle_t *pamh, const char *format,...)
 {
+	char *service = NULL;
+	char logname[256];
 	va_list args;
 
+	pam_get_item(pamh, PAM_SERVICE, (const void **) &service);
+	if (service) {
+		strncpy(logname, service, sizeof(logname));
+		logname[sizeof(logname) - 1 - strlen("(pam_unix)")] = '\0';
+		strncat(logname, "(pam_unix)", strlen("(pam_unix)"));
+	} else {
+		strncpy(logname, "pam_unix", sizeof(logname) - 1);
+	}
+
 	va_start(args, format);
-	openlog("PAM_unix", LOG_CONS | LOG_PID, LOG_AUTH);
+	openlog(logname, LOG_CONS | LOG_PID, LOG_AUTH);
 	vsyslog(err, format, args);
 	va_end(args);
 	closelog();
@@ -58,11 +69,12 @@ static int converse(pam_handle_t * pamh, int ctrl, int nargs
 		D(("returned from application's conversation function"));
 
 		if (retval != PAM_SUCCESS && on(UNIX_DEBUG, ctrl)) {
-			_log_err(LOG_DEBUG, "conversation failure [%s]"
+			_log_err(LOG_DEBUG, pamh, "conversation failure [%s]"
 				 ,pam_strerror(pamh, retval));
 		}
 	} else if (retval != PAM_CONV_AGAIN) {
-		_log_err(LOG_ERR, "couldn't obtain coversation function [%s]"
+		_log_err(LOG_ERR, pamh
+		         ,"couldn't obtain coversation function [%s]"
 			 ,pam_strerror(pamh, retval));
 	}
 	D(("ready to return from module conversation"));
@@ -94,14 +106,14 @@ int _make_remark(pam_handle_t * pamh, unsigned int ctrl
 }
 
   /*
-   * Beacause getlogin() is fucked in a weird way, and
-   * sometimes it just don't work, we reimplement it here.
+   * Beacause getlogin() is braindead and sometimes it just
+   * doesn't work, we reimplement it here.
    */
 char *PAM_getlogin(void)
 {
 	struct utmp *ut, line;
 	char *curr_tty, *retval;
-	static char curr_user[UT_NAMESIZE + 4];
+	static char curr_user[sizeof(ut->ut_user) + 4];
 
 	retval = NULL;
 
@@ -112,7 +124,7 @@ char *PAM_getlogin(void)
 		setutent();
 		strncpy(line.ut_line, curr_tty, sizeof line.ut_line);
 		if ((ut = getutline(&line)) != NULL) {
-			strncpy(curr_user, ut->ut_user, UT_NAMESIZE);
+			strncpy(curr_user, ut->ut_user, sizeof(ut->ut_user));
 			retval = curr_user;
 		}
 		endutent();
@@ -126,7 +138,8 @@ char *PAM_getlogin(void)
  * set the control flags for the UNIX module.
  */
 
-int _set_ctrl(int flags, int *remember, int argc, const char **argv)
+int _set_ctrl(pam_handle_t *pamh, int flags, int *remember, int argc,
+              const char **argv)
 {
 	unsigned int ctrl;
 
@@ -171,7 +184,8 @@ int _set_ctrl(int flags, int *remember, int argc, const char **argv)
 		}
 
 		if (j >= UNIX_CTRLS_) {
-			_log_err(LOG_ERR, "unrecognized option [%s]", *argv);
+			_log_err(LOG_ERR, pamh,
+			         "unrecognized option [%s]", *argv);
 		} else {
 			ctrl &= unix_args[j].mask;	/* for turning things off */
 			ctrl |= unix_args[j].flag;	/* for turning things on  */
@@ -221,7 +235,8 @@ static void _cleanup(pam_handle_t * pamh, void *x, int error_status)
 struct _pam_failed_auth {
 	char *user;		/* user that's failed to be authenticated */
 	char *name;		/* attempt from user with name */
-	int id;			/* uid of name'd user */
+	int uid;		/* uid of calling user */
+	int euid;		/* euid of calling process */
 	int count;		/* number of failures so far */
 };
 
@@ -233,6 +248,9 @@ static void _cleanup_failures(pam_handle_t * pamh, void *fl, int err)
 {
 	int quiet;
 	const char *service = NULL;
+	const char *ruser = NULL;
+	const char *rhost = NULL;
+	const char *tty = NULL;
 	struct _pam_failed_auth *failure;
 
 	D(("called"));
@@ -247,19 +265,29 @@ static void _cleanup_failures(pam_handle_t * pamh, void *fl, int err)
 
 			/* log the number of authentication failures */
 			if (failure->count > 1) {
-				(void) pam_get_item(pamh, PAM_SERVICE
-					      ,(const void **) &service);
-				_log_err(LOG_NOTICE
-					 ,"%d more authentication failure%s; %s(uid=%d) -> "
-					 "%s for %s service"
-					 ,failure->count - 1, failure->count == 2 ? "" : "s"
-					 ,failure->name
-					 ,failure->id
-					 ,failure->user
-				,service == NULL ? "**unknown**" : service
-				    );
+				(void) pam_get_item(pamh, PAM_SERVICE,
+						    (const void **)&service);
+				(void) pam_get_item(pamh, PAM_RUSER,
+						    (const void **)&ruser);
+				(void) pam_get_item(pamh, PAM_RHOST,
+						    (const void **)&rhost);
+				(void) pam_get_item(pamh, PAM_TTY,
+						    (const void **)&tty);
+				_log_err(LOG_NOTICE, pamh,
+				         "%d more authentication failure%s; "
+				         "logname=%s uid=%d euid=%d "
+				         "tty=%s ruser=%s rhost=%s "
+				         "%s%s",
+				         failure->count - 1, failure->count == 2 ? "" : "s",
+				         failure->name, failure->uid, failure->euid,
+				         tty ? tty : "", ruser ? ruser : "",
+				         rhost ? rhost : "",
+				         (failure->user && failure->user[0] != '\0')
+				          ? " user=" : "", failure->user
+				);
+
 				if (failure->count > UNIX_MAX_RETRIES) {
-					_log_err(LOG_ALERT
+					_log_err(LOG_ALERT, pamh
 						 ,"service(%s) ignoring max retries; %d > %d"
 						 ,service == NULL ? "**unknown**" : service
 						 ,failure->count
@@ -491,7 +519,7 @@ int _unix_verify_password(pam_handle_t * pamh, const char *name
 
 	data_name = (char *) malloc(sizeof(FAIL_PREFIX) + strlen(name));
 	if (data_name == NULL) {
-		_log_err(LOG_CRIT, "no memory for data-name");
+		_log_err(LOG_CRIT, pamh, "no memory for data-name");
 	} else {
 		strcpy(data_name, FAIL_PREFIX);
 		strcpy(data_name + sizeof(FAIL_PREFIX) - 1, name);
@@ -503,14 +531,22 @@ int _unix_verify_password(pam_handle_t * pamh, const char *name
 			/* we are not root perhaps this is the reason? Run helper */
 			D(("running helper binary"));
 			retval = _unix_run_helper_binary(pamh, p, ctrl);
+			if (pwd == NULL && !on(UNIX_AUDIT,ctrl)
+			    && retval != PAM_SUCCESS)
+			{
+				name = NULL;
+			}
 		} else {
 			D(("user's record unavailable"));
 			if (on(UNIX_AUDIT, ctrl)) {
 				/* this might be a typo and the user has given a password
 				   instead of a username. Careful with this. */
-				_log_err(LOG_ALERT, "check pass; user (%s) unknown", name);
+				_log_err(LOG_ALERT, pamh,
+				         "check pass; user (%s) unknown", name);
 			} else {
-				_log_err(LOG_ALERT, "check pass; user unknown");
+				name = NULL;
+				_log_err(LOG_ALERT, pamh,
+				         "check pass; user unknown");
 			}
 			p = NULL;
 			retval = PAM_AUTHINFO_UNAVAIL;
@@ -564,8 +600,9 @@ int _unix_verify_password(pam_handle_t * pamh, const char *name
 
 			if (new != NULL) {
 
-				new->user = x_strdup(name);
-				new->id = getuid();
+				new->user = x_strdup(name ? name : "");
+				new->uid = getuid();
+				new->euid = geteuid();
 				new->name = x_strdup(PAM_getlogin()? PAM_getlogin() : "");
 
 				/* any previous failures for this user ? */
@@ -578,23 +615,40 @@ int _unix_verify_password(pam_handle_t * pamh, const char *name
 					}
 				} else {
 					const char *service=NULL;
+					const char *ruser=NULL;
+					const char *rhost=NULL;
+					const char *tty=NULL;
+
 					(void) pam_get_item(pamh, PAM_SERVICE,
 							    (const void **)&service);
-					_log_err(LOG_NOTICE
-							,"authentication failure; %s(uid=%d) -> "
-							 "%s for %s service"
-							,new->name
-							,new->id
-							,new->user
-							,service == NULL ? "**unknown**":service
-							);
+					(void) pam_get_item(pamh, PAM_RUSER,
+							    (const void **)&ruser);
+					(void) pam_get_item(pamh, PAM_RHOST,
+							    (const void **)&rhost);
+					(void) pam_get_item(pamh, PAM_TTY,
+							    (const void **)&tty);
+
+					_log_err(LOG_NOTICE, pamh,
+					         "authentication failure; "
+					         "logname=%s uid=%d euid=%d "
+					         "tty=%s ruser=%s rhost=%s "
+					         "%s%s",
+					         new->name, new->uid, new->euid,
+					         tty ? tty : "",
+					         ruser ? ruser : "",
+					         rhost ? rhost : "",
+					         (new->user && new->user[0] != '\0')
+					          ? " user=" : "",
+					         new->user
+					);
 					new->count = 1;
 				}
 
 				pam_set_data(pamh, data_name, new, _cleanup_failures);
 
 			} else {
-				_log_err(LOG_CRIT, "no memory for failure recorder");
+				_log_err(LOG_CRIT, pamh,
+				         "no memory for failure recorder");
 			}
 		}
 	}
@@ -650,7 +704,7 @@ int _unix_read_password(pam_handle_t * pamh
 		retval = pam_get_item(pamh, authtok_flag, (const void **) &item);
 		if (retval != PAM_SUCCESS) {
 			/* very strange. */
-			_log_err(LOG_ALERT
+			_log_err(LOG_ALERT, pamh
 				 ,"pam_get_item returned error to unix-read-password"
 			    );
 			return retval;
@@ -721,7 +775,7 @@ int _unix_read_password(pam_handle_t * pamh
 						}
 					}
 				} else {
-					_log_err(LOG_NOTICE
+					_log_err(LOG_NOTICE, pamh
 						 ,"could not recover authentication token");
 				}
 
@@ -741,7 +795,8 @@ int _unix_read_password(pam_handle_t * pamh
 
 	if (retval != PAM_SUCCESS) {
 		if (on(UNIX_DEBUG, ctrl))
-			_log_err(LOG_DEBUG, "unable to obtain a password");
+			_log_err(LOG_DEBUG, pamh,
+			         "unable to obtain a password");
 		return retval;
 	}
 	/* 'token' is the entered password */
@@ -757,7 +812,7 @@ int _unix_read_password(pam_handle_t * pamh
 					      ,(const void **) &item))
 		    != PAM_SUCCESS) {
 
-			_log_err(LOG_CRIT, "error manipulating password");
+			_log_err(LOG_CRIT, pamh, "error manipulating password");
 			return retval;
 
 		}
@@ -769,7 +824,8 @@ int _unix_read_password(pam_handle_t * pamh
 
 		retval = pam_set_data(pamh, data_name, (void *) token, _cleanup);
 		if (retval != PAM_SUCCESS) {
-			_log_err(LOG_CRIT, "error manipulating password data [%s]"
+			_log_err(LOG_CRIT, pamh
+			         ,"error manipulating password data [%s]"
 				 ,pam_strerror(pamh, retval));
 			_pam_delete(token);
 			return retval;
