@@ -1,5 +1,5 @@
 /*
- * $Id: unix_chkpwd.c,v 1.2 1999/07/08 05:02:11 morgan Exp $
+ * $Id: unix_chkpwd.c,v 1.2 2000/11/19 23:54:05 agmorgan Exp $
  *
  * This program is designed to run setuid(root) or with sufficient
  * privilege to read all of the unix password databases. It is designed
@@ -13,11 +13,7 @@
  *
  */
 
-#define _BSD_SOURCE
-#ifdef linux
-# define _GNU_SOURCE
-# include <features.h>
-#endif
+#include <security/_pam_aconf.h>
 
 #include <stdarg.h>
 #include <stdio.h>
